Learn R Programming

randomUniformForest (version 1.1.2)

rUniformForest.grow: Add trees to a random Uniform Forest

Description

add more trees to the ensemble model

Usage

rUniformForest.grow(object, X, Y = NULL, ntree = 100, threads = "auto")

Arguments

Value

  • an object of class randomUniformForest, containing new and old trees.

Details

rUniformForest.grow allows both to add new trees or new model (by adding trees on a new matrix and training responses) built on the same bases than the former. Note that with formula, only new trees can be added, not new model.

See Also

rUniformForest.big, rUniformForest.combine

Examples

Run this code
# data(iris)
# iris.rf <- randomUniformForest(Species ~ ., iris, ntree = 50, threads = 1, BreimanBounds = FALSE)
# iris.rf <- rUniformForest.grow(iris.rf, iris, ntree = 20, threads = 1)
# iris.rf

Run the code above in your browser using DataLab