an object of class randomUniformForest, containing new and old trees.
Details
rUniformForest.grow allows both to add new trees or new model (by adding trees on a new matrix and training responses) built on
the same bases than the former. Note that with formula, only new trees can be added, not new model.